#!/usr/bin/env python

# Copyright 2013 Canonical Ltd. All rights reserved.
# Author: Brad Marshall <brad.marshall@canonical.com>

import glob
import os
import sys
import shutil
import subprocess
import socket
import stat
import pwd
import grp
import json
import yaml
import distutils.sysconfig

local_copy = os.path.join(
    os.path.dirname(os.path.abspath(os.path.dirname(__file__))),
    "lib", "charm-helpers")
if os.path.exists(local_copy) and os.path.isdir(local_copy):
    sys.path.insert(0, local_copy)


from charmhelpers.core.host import (
    mkdir,
    service,
    service_start,
    service_stop,
    service_restart,
    write_file,
)

from charmhelpers.core.hookenv import (
    Hooks,
    config,
    open_port,
    close_port,
    relation_get,
    relation_set,
    relation_id,
    remote_unit,
    local_unit,
)

from charmhelpers.fetch import (
    apt_install,
)

unit_name = local_unit().split("/")[0]
hostname = socket.getfqdn()
basedir = "/srv/" + unit_name
rundir = basedir + "/run"
logdir = basedir + "/logs"
staticdir = basedir + "/moin_static"
wikiconfig = basedir + "/wikiconfig.py"
wsgipy = basedir + "/wsgi.py"
loggingconf = basedir + "/logging.conf"
xapian_initialised = basedir + "/xapian_is_initialized"
gunicornconf = "/etc/gunicorn.d/" + unit_name + ".conf"
wiki_name = config().get('wiki_name')
admin_name = config().get('admin_name')
languages = config().get('languages')
loglevel = config().get('loglevel')
xapian_search = config().get('xapian_search')
listen_port = config().get('listen_port')
use_openid = config().get('use_openid')
openidrp_authorized_teams = config().get('openidrp_authorized_teams')
extra_settings = config().get('extra_settings')

hook_dir = os.path.abspath(os.path.dirname(__file__))
charm_dir = os.path.dirname(hook_dir)

required_pkgs = [
    'python-moinmoin',
    'python-flup',
    'gunicorn',
    'python-eventlet',
    'python-tz',
    'unzip',
    'python-openid',
    'python-xapian',
    'python-xappy',
    'python-docutils',
    'python-jinja2'
]

hooks = Hooks()


@hooks.hook()
def install():
    apt_install(required_pkgs, options=['--force-yes'])
    if os.path.exists('/etc/moin/farmconfig.py'):
        os.remove('/etc/moin/farmconfig.py')
    if not os.path.exists(rundir):
        os.makedirs(rundir)
    if not os.path.exists(logdir):
        os.makedirs(logdir)
    if not os.path.exists(basedir + "/data"):
        shutil.copytree('/usr/share/moin/data', basedir + "/data")
    if not os.path.exists(basedir + "/underlay"):
        shutil.copytree('/usr/share/moin/underlay', basedir + "/underlay")
    if not os.path.exists(staticdir):
        shutil.copytree('/usr/share/moin/htdocs', staticdir)
    open_port(listen_port)

@hooks.hook("config-changed")
def config_changed():
    # See if the port has changed
    if (os.path.exists(gunicornconf)):
        for line in open(gunicornconf):
            if "--bind" in line:
                port = line.split("=")[1].split(":")[1].rstrip("',\r\n")
                break
        if (listen_port != port):
            close_port(port)
            open_port(listen_port)

    # Create files from templates
    from jinja2 import Environment, FileSystemLoader
    template_env = Environment(
        loader=FileSystemLoader(os.path.join(os.environ['CHARM_DIR'],
                                'templates')))
    templ_vars = {
        'basedir': basedir,
        'unit_name': unit_name,
        'wiki_name': wiki_name,
        'admin_name': admin_name,
        'languages': languages,
        'loglevel': loglevel,
        'xapian_search': xapian_search,
        'listen_port': listen_port,
        'use_openid': use_openid,
        'openidrp_authorized_teams': openidrp_authorized_teams,
        'extra_settings': extra_settings,
    }

    # gunicorn.conf.tmpl  logging.conf.tmpl  wikiconfig.py.tmpl  wsgi.py.tmpl
    wikiconfigtemplate = \
        template_env.get_template('wikiconfig.py.tmpl').render(templ_vars)
    with open(wikiconfig, 'w') as wikiconfig_config:
        wikiconfig_config.write(str(wikiconfigtemplate))
    wsgipytemplate = \
        template_env.get_template('wsgi.py.tmpl').render(templ_vars)
    with open(wsgipy, 'w') as wsgipy_config:
        wsgipy_config.write(str(wsgipytemplate))
    loggingconftemplate = \
        template_env.get_template('logging.conf.tmpl').render(templ_vars)
    with open(loggingconf, 'w') as loggingconf_config:
        loggingconf_config.write(str(loggingconftemplate))
    gunicornconftemplate = \
        template_env.get_template('gunicorn.conf.tmpl').render(templ_vars)
    with open(gunicornconf, 'w') as gunicornconf_config:
        gunicornconf_config.write(str(gunicornconftemplate))

    # Handle xapian search updates
    if (xapian_search == "True"):
        if not os.path.exists(xapian_initialised):
            subprocess.call(['moin', "--config-dir=%s/" % (basedir), 'index',
                            'build', '--mode=add'])
            open(xapian_initialised, 'w').close()
        else:
            subprocess.call(['moin', "--config=dir=%s/" % (basedir), 'index',
                            'build', '--mode=rebuild'])

    # Install languages
    for lang in languages.split():
        subprocess.call(['python', '-m', 'MoinMoin.packages', 'i',
            "%s/underlay/pages/LanguageSetup/attachments/%s--all_pages.zip"
            % (basedir, lang)], cwd=basedir)

    # fix perms to be owned by root:www-data and g+rwX
    root_uid = pwd.getpwnam('root').pw_uid
    www_gid = grp.getgrnam('www-data').gr_gid
    for root, dirs, files in os.walk(basedir):
        for d in dirs:
            os.chown(os.path.join(root, d), root_uid, www_gid)
            os.chmod(os.path.join(root, d), 0775)
        for f in files:
            os.chown(os.path.join(root, f), root_uid, www_gid)
            os.chmod(os.path.join(root, f), 0664)

    # if we are using openid make sure files are in place
    openid_python_path = "/usr/share/pyshared/MoinMoin/auth/openidrp_ext"
    dist_dir = distutils.sysconfig.get_python_lib()
    if (use_openid == "True"):
	# Copy the openidrp.py file in place
	shutil.copyfile("files/openidrp.py",
			"%s/MoinMoin/auth/openidrp.py" % (dist_dir))
        # Copy the openidrp_teams.py file in place
        shutil.copyfile("files/openidrp_teams.py",
                        "%s/openidrp_teams.py" % (openid_python_path))
        # Only copy and symlink the teams.py in place if we don't have it
        if not os.path.exists("%s/teams.py" % (openid_python_path)):
            shutil.copyfile("files/teams.py", "%s/teams.py"
                            % (openid_python_path))
            os.symlink("%s/teams.py" % (openid_python_path),
                       "%s/MoinMoin/auth/openidrp_ext/teams.py" % (dist_dir))

    service_restart('gunicorn')


@hooks.hook("website-relation-joined")
def website_relation_join():
    relation_set(port=listen_port, hostname=hostname)


@hooks.hook("upgrade-charm")
def upgrade_charm():
    install()


@hooks.hook("start")
def start():
    service_start('gunicorn')


@hooks.hook("stop")
def stop():
    service_stop('gunicorn')


if __name__ == "__main__":
    hooks.execute(sys.argv)
